#region Copyright Morrison Shwartz, Inc - Author: Jonah Stagner 
/////////////////////////////////////////////////////////////////////////
// Copyright Morrison Schwartz, Inc 2003-2004, All rights reserved     //
//---------------------------------------------------------------------//
//Author: Jonah L. Stagner                                             //
//---------------------------------------------------------------------//
// This product is protected by US Copyright laws and international    //
// treaties.  For more information, see the End User License Agreement //
// in the application installation folder.                             //
//---------------------------------------------------------------------//
// Visit us on the web at: http://www.ms-inc.net                       //
/////////////////////////////////////////////////////////////////////////
#endregion


using System;
using System.IO;
using System.Collections;
using System.Windows.Forms;

using SharpTools;
using SharpTools.DOM;
using SharpTools.Settings;
using SharpTools.PluginInterface;

namespace Add_Copyright_Notices
{

	/// <summary>
	/// This SharpTools plugin adds a custom Copyright notice to every code 
	/// file in the current project
	/// </summary>
	public class AddCopyright : ISharpToolsPlugin	
	{

		#region Private data members 

		private SharpTools.Toolbar			toolbar;
		private SharpTools.ToolbarButton	cmdRun;
		private PropertyPageBase			propertyPage;

		#endregion 

		#region Constructor 

		public AddCopyright()
		{
			// Stub.  A public default constructor is required by SharpTools 
			// so that the assembly can be queried using Reflection to discover
			// all public classes that implement the SharpToolsPlugin interface.
		}

		#endregion 

		#region ISharpToolsPlugin Members 

		#region ISharpToolsPlugin properties 

		// The description that will be displayed to the user in the SharpTools Configuration dialog
		public string Description
		{
			get { return null; }
		}

		// The name of the plugin.  This name will be displayed to the user in the SharpTools Configuration dialog.
		public string Name
		{
			get { return "Add Copyright Notice"; }
		}

		// A string that uniquely identifies this plugin.  Can be any unique string, but since it 
		// is otherwise meaningless, a string representation of a GUID is usually used.
		public string ID
		{
			get { return "9418EA5A-6CCC-41b1-87D5-435F607A98A4"; }
		}

		#endregion 

		#region ISharpToolsPlugin methods 

		// This method is called by SharpTools when the plugin is loaded and is ready to create it's VS.NET
		// user interface.
		public void OnConnect()
		{
	
			// Create a Toolbar specific to this plugin
			toolbar = new SharpTools.Toolbar( "AddCopyrightPlugin" );

			// Add a button to the toolbar which, when clicked, will 
			// add the custom header to the code files
			cmdRun = (SharpTools.ToolbarButton) toolbar.Controls.Add( ControlType.Button, "Run" );
			cmdRun.Caption = "Add Copyright";
			cmdRun.Tooltip = "Add custom header to all code files";
			cmdRun.ButtonStyle = ButtonStyle.Caption;
			cmdRun.Click += new EventHandler(cmdRun_Click);

			// Create the property page and register it with SharpTools
			propertyPage = new HeaderProperties();
			Globals.PropertyPageManager.RegisterPropertyPage( propertyPage );

		}

		// This method is called by SharpTools when the plugin needs to unload.  This may happen when the user
		// exits the VS.NET IDE, when they unload SharpTools, or when they unload your plugin specifically.
		public void OnDisconnect(SharpTools.PluginInterface.DisconnectMode disconnectMode)
		{
			
			Globals.PropertyPageManager.UnRegisterPropertyPage( propertyPage );

		}

		#endregion 

		#endregion

		#region Private utility methods 

		private string GetHeaderText()
		{

			// Retrieve this plugin's Properties collection from the SharpTools property storage
			Properties properties = (Properties) Globals.Properties.GetProperty( "Add Copyright Notice", new Properties() );

			// Retrieve the custom header text from the Properties collection
			string headerText = properties.GetProperty( "HeaderText", "" );

			// Return the custom header text, ensuring that it has a linefeed at the end
			return headerText + "\r\n";

		}

		#endregion 

		#region Toolbar button click handlers 

		private void AddHeader( ProjectItem item, string headerText )
		{

			if( item.CodeFile != null )
			{

				// Get a reference to the code file
				CodeFile file = item.CodeFile;

				// Open the file if it is not already open
				if( !file.IsOpen )
					file.Open();

				// Bring the file to focus
				file.Activate();

				// Get an EditPoint for the start of the file.  An EditPoint allows the 
				// insertion, deletion, and searching of text.
				EditPoint point = file.Start.CreateEditPoint();
					
				// Try to ensure that the header text does not already exist
				if( !point.FindPattern( headerText, FindOptions.None ) )
				{

					// Insert header text at the beginning of the file and display to the user.
					point.Insert( headerText );
					point.MoveToAbsoluteOffset( 0 );
					point.TryToShow( ShowMode.Top );

				}

			}

			// Traverse subfolders to make sure that we don't miss any files
			foreach( ProjectItem child in item.ProjectItems )
				AddHeader( child, headerText );

		}

		private void cmdRun_Click(object sender, EventArgs e)
		{

			// Get a reference to the currently active project
			Project active = Globals.ActiveProject;
			if( active == null )
			{
				MessageBox.Show( "Please load a project before running this plugin", "Add Copyright Notice" );
				return;
			}

			char[] TrimChars = { ' ', '\t', '\r', '\n' };

			// Retrieve custom header text from the SharpTools property storage
			string headerText = GetHeaderText();
			if( headerText.Trim( TrimChars ) == "" )
			{
				MessageBox.Show( "The header text has not been set.  Please use the SharpTools preferences dialog to set the header text", "Add Copyright Notice" );
				return;
			}

			string alert = "Click the OK button to insert the following text into all source code files in your project:\r\n\r\n" + headerText;
			if( MessageBox.Show( alert, "Add Copyright to all files?", MessageBoxButtons.OKCancel ) != DialogResult.OK )
				return;

			// Iterate all ProjectItem object in the current project, adding header text to code files
			foreach( ProjectItem item in active.ProjectItems )
				AddHeader( item, headerText );

		}

		#endregion 

	}

}
